-- The Macaulay 2 code for Daniel Barath, Zuzana Kukelova, Relative Pose from SIFT Features, ECCV 2022
-- The code generates constraints relating elements of  A to the scales and rotations of two features in two images 
-- The code can be run online on:
-- https://www.unimelb-macaulay2.cloud.edu.au/#home


R = QQ[a1,a2,a3,a4,              -- affinity   A
       c1,s1,c2,s2,q1,q2,        -- rotations(sin,cos)+scales (SIFT)
       q,                         -- q2^2/q1^2
       w,                        -- shear
       qu,qv,                     -- scale 
       h1,h2,h3,h4,h5,h6,h7,h8,h9,   -- elements of f3
       u1,u2,v1,v2                  -- point coreespondences
      ]       

-- Form constraints relating elements of  A to the scales and rotations of two features in two images 

-- Constraints from the decomposition of A
g1 = c2*c1*qu-c2*s1*w+s2*s1*qv-a1
g2 = c2*s1*qu+c2*c1*w-s2*c1*qv-a2
g3 = s2*c1*qu-s2*s1*w-c2*s1*qv-a3
g4 = s2*s1*qu+s2*c1*w+c2*c1*qv-a4

-- Constraints from scales
g5 = qu*qv-q^2
g6 = q1*q-q2

-- Constraints from sin and cos
g7 = c1^2+s1^2-1
g8 = c2^2+s2^2-1

-- Constraints from oriented circles
f1 = c1 * q1 * a1 + s1 * q1 * a2 - c2 * q2
f2 = c1 * q1 * a3 + s1 * q1 * a4 - s2 * q2

-- Constraint from scales detA = q^2
f3 = a1*a4-a2*a3-q^2

-- Ideal
I = ideal(g1|g2|g3|g4|g5|g6|g7|g8|f1|f2|f3)

-- Saturate the ideal
S = ideal(q1)

I1 = saturate(I,S)

-- Elimination ideal
I2 = eliminate({qu,qv,w,q1,q2},I1)

p = mingens I2